@@ -1,33 +1,34 @@
 """
-    flask_security.views
-    ~~~~~~~~~~~~~~~~~~~~
+flask_security.views
+~~~~~~~~~~~~~~~~~~~~
 
-    Flask-Security views module
+Flask-Security views module
 
-    :copyright: (c) 2012 by Matt Wright.
-    :copyright: (c) 2019-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2012 by Matt Wright.
+:copyright: (c) 2019-2025 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 
-    CSRF is tricky. By default all our forms have CSRF protection built in via
-    Flask-WTF. This is regardless of authentication method or whether the request
-    is Form or JSON based. Form-based 'just works' since when rendering the form
-    (on GET), the CSRF token is automatically populated.
-    We want to handle:
-        - JSON requests where CSRF token is in a header (e.g. X-CSRF-Token)
-        - Option to skip CSRF when using a token to authenticate (rather than session)
-          (CSRF_PROTECT_MECHANISMS)
-        - Option to skip CSRF for 'login'/unauthenticated requests
-          (CSRF_IGNORE_UNAUTH_ENDPOINTS)
-    This is complicated by the fact that the only way to disable form CSRF is to
-    pass in meta={csrf: false} at form instantiation time.
+CSRF is tricky. By default all our forms have CSRF protection built in via
+Flask-WTF. This is regardless of authentication method or whether the request
+is Form or JSON based. Form-based 'just works' since when rendering the form
+(on GET), the CSRF token is automatically populated.
+We want to handle:
+    - JSON requests where CSRF token is in a header (e.g. X-CSRF-Token)
+    - Option to skip CSRF when using a token to authenticate (rather than session)
+      (CSRF_PROTECT_MECHANISMS)
+    - Option to skip CSRF for 'login'/unauthenticated requests
+      (CSRF_IGNORE_UNAUTH_ENDPOINTS)
+This is complicated by the fact that the only way to disable form CSRF is to
+pass in meta={csrf: false} at form instantiation time.
 
-    Be aware that for CSRF to work, caller MUST pass in session cookie. So
-    for pure API, and no session cookie - there is no way to support CSRF-Login
-    so app must set CSRF_IGNORE_UNAUTH_ENDPOINTS (or use CSRF/session cookie for logging
-    in then once they have a token, no need for cookie).
+Be aware that for CSRF to work, caller MUST pass in session cookie. So
+for pure API, and no session cookie - there is no way to support CSRF-Login
+so app must set CSRF_IGNORE_UNAUTH_ENDPOINTS (or use CSRF/session cookie for logging
+in then once they have a token, no need for cookie).
 
 """
 
+from __future__ import annotations
 from functools import partial
 import time
 import typing as t
@@ -35,6 +36,7 @@ import typing as t
 from flask import (
     Blueprint,
     after_this_request,
+    current_app,
     jsonify,
     request,
     session,
@@ -42,6 +44,8 @@ from flask import (
 from flask_login import current_user
 
 from .changeable import change_user_password
+from .change_email import change_email, change_email_confirm
+from .change_username import change_username
 from .confirmable import (
     confirm_email_token_status,
     confirm_user,
@@ -62,10 +66,12 @@ from .forms import (
     TwoFactorVerifyCodeForm,
     TwoFactorSetupForm,
     TwoFactorRescueForm,
+    UsernameRecoveryForm,
 )
 from .passwordless import login_token_status, send_login_instructions
 from .proxies import _security, _datastore
 from .quart_compat import get_quart_status
+from .signals import tf_profile_changed
 from .unified_signin import (
     us_signin,
     us_signin_send_code,
@@ -79,6 +85,7 @@ from .recoverable import (
     reset_password_token_status,
     send_reset_password_instructions,
     update_password,
+    send_username_recovery_email,
 )
 from .registerable import register_user, register_existing
 from .recovery_codes import mf_recovery, mf_recovery_codes
@@ -96,6 +103,7 @@ from .twofactor import (
 from .utils import (
     base_render_json,
     check_and_update_authn_fresh,
+    check_and_get_token_status,
     config_value as cv,
     do_flash,
     get_identity_attributes,
@@ -105,10 +113,11 @@ from .utils import (
     get_post_register_redirect,
     get_post_verify_redirect,
     get_request_attr,
+    get_within_delta,
     get_url,
+    handle_already_auth,
     hash_password,
     is_user_authenticated,
-    json_error_response,
     localize_callback,
     login_user,
     logout_user,
@@ -152,36 +161,15 @@ def _ctx(endpoint):
     return _security._run_ctx_processor(endpoint)
 
 
-@unauth_csrf(fall_through=True)
-def login() -> "ResponseValue":
-    """View function for login view
-
-    Allow already authenticated users. For GET this is useful for
-    single-page-applications on refresh - session still active but need to
-    access user info and csrf-token.
-    For POST - redirects to POST_LOGIN_VIEW (forms) or returns 400 (json).
-    """
+@unauth_csrf()
+def login() -> ResponseValue:
+    """View function for login view"""
     form = t.cast(LoginForm, build_form_from_request("login_form"))
 
     if is_user_authenticated(current_user):
-        # Just redirect current_user to POST_LOGIN_VIEW.
-        # While its tempting to try to logout the current user and login the
-        # new requested user - that simply doesn't work with CSRF.
-
-        # This does NOT use get_post_login_redirect() so that it doesn't look at
-        # 'next' - which can cause infinite redirect loops
-        # (see test_common::test_authenticated_loop)
-        if _security._want_json(request):
-            if request.method == "POST":
-                payload = json_error_response(
-                    errors=get_message("ANONYMOUS_USER_REQUIRED")[0]
-                )
-                return _security._render_json(payload, 400, None, None)
-            else:
-                form.user = current_user
-                return base_render_json(form)
-        else:
-            return redirect(get_url(cv("POST_LOGIN_VIEW")))
+        return handle_already_auth(
+            form, payload={"identity_attributes": get_identity_attributes()}
+        )
 
     # Clean out any potential old session info - in case of previous
     # aborted 2FA attempt.
@@ -216,6 +204,11 @@ def login() -> "ResponseValue":
             fields_to_squash["username"] = dict(replace_msg="GENERIC_AUTHN_FAILED")
         form_errors_munge(form, fields_to_squash)
 
+    if request.method == "GET":
+        # set CSRF COOKIE if configured. This is the equivalent of forms and
+        # base_render_json always sending the csrf_token
+        session["fs_cc"] = "set"
+
     if _security._want_json(request):
         payload = {
             "identity_attributes": get_identity_attributes(),
@@ -227,7 +220,9 @@ def login() -> "ResponseValue":
         and cv("REQUIRES_CONFIRMATION_ERROR_VIEW")
         and not cv("RETURN_GENERIC_RESPONSES")
     ):
+        # Validation failed BECAUSE user needs to confirm
         assert form.user_authenticated
+        assert form.email.data  # email_required validator
         do_flash(*get_message("CONFIRMATION_REQUIRED"))
         return redirect(
             get_url(
@@ -291,15 +286,15 @@ def logout():
 
 
 @anonymous_user_required
-@unauth_csrf(fall_through=True)
-def register() -> "ResponseValue":
+@unauth_csrf()
+def register() -> ResponseValue:
     """View function which handles a registration request."""
 
     # For some unknown historic reason - if you don't require confirmation
     # (via email) then you need to type in your password twice. That might
     # make sense if you can't reset your password but in modern (2020) UX models
     # don't ask twice.
-    if _security.confirmable or request.is_json:
+    if (_security.confirmable or request.is_json) and _security._use_confirm_form:
         form_name = "confirm_register_form"
     else:
         form_name = "register_form"
@@ -350,7 +345,7 @@ def register() -> "ResponseValue":
     )
 
 
-@unauth_csrf(fall_through=True)
+@unauth_csrf()
 def send_login():
     """View function that sends login instructions for passwordless login"""
     form = build_form_from_request("passwordless_login_form")
@@ -408,7 +403,7 @@ def token_login(token):
     return redirect(get_post_login_redirect())
 
 
-@unauth_csrf(fall_through=True)
+@unauth_csrf()
 def send_confirmation():
     """View function which sends confirmation instructions (/confirm)."""
     form = t.cast(
@@ -522,7 +517,7 @@ def confirm_email(token):
 
 
 @anonymous_user_required
-@unauth_csrf(fall_through=True)
+@unauth_csrf()
 def forgot_password():
     """View function that handles a forgotten password request (/reset)."""
     form = t.cast(ForgotPasswordForm, build_form_from_request("forgot_password_form"))
@@ -570,7 +565,7 @@ def forgot_password():
 
 
 @anonymous_user_required
-@unauth_csrf(fall_through=True)
+@unauth_csrf()
 def reset_password(token):
     """View function that handles a reset password request (/reset/<token>).
 
@@ -725,7 +720,7 @@ def change_password():
     )
 
 
-@unauth_csrf(fall_through=True)
+@unauth_csrf()
 def two_factor_setup():
     """View function for two-factor setup.
 
@@ -747,9 +742,9 @@ def two_factor_setup():
     """
     form = t.cast(TwoFactorSetupForm, build_form_from_request("two_factor_setup_form"))
 
-    if not is_user_authenticated(current_user):
+    changing = is_user_authenticated(current_user)
+    if not changing:
         # This is the initial login case
-        # We can also get here from setup if they want to change TODO: how?
         if not all(k in session for k in ["tf_user_id", "tf_state"]) or session[
             "tf_state"
         ] not in ["setup_from_login", "validating_profile"]:
@@ -794,15 +789,25 @@ def two_factor_setup():
                 return base_render_json(form)
 
         # Regenerate the TOTP secret on every call of 2FA setup
-        totp = _security._totp_factory.generate_totp_secret()
+        totp = _security.totp_factory.generate_totp_secret()
         phone = form.phone.data if pm == "sms" else None
         session["tf_totp_secret"] = totp
         session["tf_primary_method"] = pm
         session["tf_state"] = "validating_profile"
+        # currently - state_token only works for changing TFA - not initial login
+        state_token = None
+        if changing:
+            state = {
+                "totp_secret": totp,
+                "method": pm,
+                "phone": phone,
+            }
+            state_token = _security.tf_setup_serializer.dumps(state)
         json_response = {
-            "tf_state": "validating_profile",
+            "tf_state": "validating_profile",  # deprecated in 5.5.0
             "tf_primary_method": pm,  # old
             "tf_method": pm,
+            "tf_state_token": state_token,
         }
         if phone:
             #  TODO dont save here - wait until complete
@@ -829,7 +834,7 @@ def two_factor_setup():
 
         qrcode_values = dict()
         if pm == "authenticator":
-            authr_setup_values = _security._totp_factory.fetch_setup_values(totp, user)
+            authr_setup_values = _security.totp_factory.fetch_setup_values(totp, user)
             # Add all the values used in qrcode to json response
             json_response["tf_authr_key"] = authr_setup_values["key"]
             json_response["tf_authr_username"] = authr_setup_values["username"]
@@ -853,14 +858,14 @@ def two_factor_setup():
             primary_method=localize_callback(
                 _setup_methods_xlate[getattr(user, "tf_primary_method", None)]
             ),
+            changing=changing,
+            state_token=state_token,
             **qrcode_values,
             **_ctx("tf_setup"),
         )
 
     # We get here on GET and POST with failed validation.
-    # For things like phone number - we've already done one POST
-    # that succeeded and now it failed - so retain the initial info
-    choices = cv("TWO_FACTOR_ENABLED_METHODS")
+    choices = cv("TWO_FACTOR_ENABLED_METHODS")[:]
     if (not cv("TWO_FACTOR_REQUIRED")) and user.tf_primary_method is not None:
         choices.insert(0, "disable")
 
@@ -881,16 +886,91 @@ def two_factor_setup():
         two_factor_setup_form=form,
         two_factor_verify_code_form=code_form,
         choices=choices,
-        chosen_method=form.setup.data,
+        chosen_method=None,
         primary_method=localize_callback(
             _setup_methods_xlate[getattr(user, "tf_primary_method", None)]
         ),
+        changing=changing,
+        state_token=None,
         two_factor_required=cv("TWO_FACTOR_REQUIRED"),
         **_ctx("tf_setup"),
     )
 
 
-@unauth_csrf(fall_through=True)
+@auth_required(lambda: cv("API_ENABLED_METHODS"))
+def two_factor_setup_validate(token: str) -> ResponseValue:
+    """
+    Validate new setup.
+    The token is the state variable which is signed and timed
+    and contains all the state that once confirmed will be stored in the user record.
+    Unlike the code in two_factor_token_validation - this works w/o a session.
+    It also is JUST for setting up/changing two factor for an authenticated user.
+    """
+    form = t.cast(
+        TwoFactorVerifyCodeForm, build_form_from_request("two_factor_verify_code_form")
+    )
+
+    expired, invalid, state = check_and_get_token_status(
+        token, "tf_setup", get_within_delta("TWO_FACTOR_SETUP_WITHIN")
+    )
+    if invalid:
+        m, c = get_message("API_ERROR")
+    if expired:
+        m, c = get_message(
+            "TWO_FACTOR_SETUP_EXPIRED", within=cv("TWO_FACTOR_SETUP_WITHIN")
+        )
+    if invalid or expired:
+        tf_clean_session()  # until we completely remove session based setup/state
+        if _security._want_json(request):
+            form.form_errors.append(m)
+            return base_render_json(form, include_user=False)
+        do_flash(m, c)
+        return redirect(url_for_security("two_factor_setup"))
+
+    totp_secret = state["totp_secret"]
+    method = state["method"]
+    phone = state["phone"]
+    form.tf_totp_secret = totp_secret
+    form.primary_method = method
+    form.user = current_user
+
+    if form.validate_on_submit():
+        tf_clean_session()  # until we completely remove session based setup/state
+        after_this_request(view_commit)
+        _datastore.tf_set(current_user, method, totp_secret, phone)
+        # TODO: should validity cookie be removed? extended? left alone?
+        # Currently - leave it alone - meaning cookie still set.
+
+        tf_profile_changed.send(
+            current_app._get_current_object(),  # type: ignore[attr-defined]
+            _async_wrapper=current_app.ensure_sync,
+            user=current_user,
+            method=method,
+        )
+
+        if _security._want_json(request):
+            return base_render_json(
+                form,
+                include_user=False,
+                additional=dict(
+                    tf_method=method,
+                    tf_primary_method=method,
+                    tf_phone=current_user.tf_phone_number,
+                ),
+            )
+        else:
+            do_flash(*get_message("TWO_FACTOR_CHANGE_METHOD_SUCCESSFUL"))
+            return redirect(get_url(cv("TWO_FACTOR_POST_SETUP_VIEW")))
+
+    # Code not correct/outdated.
+    if _security._want_json(request):
+        return base_render_json(form, include_user=False)
+    m, c = get_message("TWO_FACTOR_INVALID_TOKEN")
+    do_flash(m, c)
+    return redirect(url_for_security("two_factor_setup"))
+
+
+@unauth_csrf()
 def two_factor_token_validation():
     """View function for two-factor token validation
 
@@ -998,7 +1078,7 @@ def two_factor_token_validation():
 
 
 @anonymous_user_required
-@unauth_csrf(fall_through=True)
+@unauth_csrf()
 def two_factor_rescue():
     """Function that handles a situation where user can't
     enter his two-factor validation code
@@ -1068,6 +1148,41 @@ def two_factor_rescue():
     )
 
 
+@anonymous_user_required
+@unauth_csrf()
+def recover_username():
+    """View function for username recovery"""
+
+    form = t.cast(
+        UsernameRecoveryForm, build_form_from_request("username_recovery_form")
+    )
+
+    if form.validate_on_submit():
+        send_username_recovery_email(form.user)
+
+        if _security._want_json(request):
+            return base_render_json(form, include_user=False)
+
+        do_flash(*get_message("USERNAME_RECOVERY_REQUEST"))
+
+        return redirect(url_for_security("login"))
+    elif request.method == "POST" and cv("RETURN_GENERIC_RESPONSES"):
+        rinfo = dict(email=dict())
+        form_errors_munge(form, rinfo)
+        if not form.errors:
+            if not _security._want_json(request):
+                do_flash(*get_message("USERNAME_RECOVERY_REQUEST"))
+
+    if _security._want_json(request):
+        return base_render_json(form, include_user=False)
+
+    return _security.render_template(
+        cv("USERNAME_RECOVERY_TEMPLATE"),
+        username_recovery_form=form,
+        **_ctx("recover_username"),
+    )
+
+
 def create_blueprint(app, state, import_name):
     """Creates the security extension blueprint"""
 
@@ -1153,6 +1268,11 @@ def create_blueprint(app, state, import_name):
             methods=["GET", "POST"],
             endpoint="two_factor_setup",
         )(two_factor_setup)
+        bp.route(
+            two_factor_setup_url + slash_url_suffix(two_factor_setup_url, "<token>"),
+            methods=["POST"],
+            endpoint="two_factor_setup_validate",
+        )(two_factor_setup_validate)
         bp.route(
             two_factor_token_validation_url,
             methods=["GET", "POST"],
@@ -1180,6 +1300,14 @@ def create_blueprint(app, state, import_name):
             endpoint="reset_password",
         )(reset_password)
 
+    if state.username_recovery:
+        username_recovery_url = cv("USERNAME_RECOVERY_URL", app=app)
+        bp.route(
+            username_recovery_url,
+            methods=["GET", "POST"],
+            endpoint="recover_username",
+        )(recover_username)
+
     if state.changeable:
         bp.route(
             cv("CHANGE_URL", app=app),
@@ -1187,6 +1315,26 @@ def create_blueprint(app, state, import_name):
             endpoint="change_password",
         )(change_password)
 
+    if state.change_email:
+        change_email_url = cv("CHANGE_EMAIL_URL", app=app)
+        bp.route(
+            change_email_url,
+            methods=["GET", "POST"],
+            endpoint="change_email",
+        )(change_email)
+        bp.route(
+            change_email_url + slash_url_suffix(change_email_url, "<token>"),
+            methods=["GET"],
+            endpoint="change_email_confirm",
+        )(change_email_confirm)
+
+    if state.change_username:
+        bp.route(
+            cv("CHANGE_USERNAME_URL", app=app),
+            methods=["GET", "POST"],
+            endpoint="change_username",
+        )(change_username)
+
     if state.confirmable:
         confirm_url = cv("CONFIRM_URL", app=app)
         bp.route(confirm_url, methods=["GET", "POST"], endpoint="send_confirmation")(
