@@ -1,38 +1,38 @@
 """
-    flask_security.webauthn
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+flask_security.webauthn
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    Flask-Security WebAuthn module
+Flask-Security WebAuthn module
 
-    :copyright: (c) 2021-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2021-2025 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 
-    This implements support for webauthn/FIDO2 Level 2 using the py_webauthn package.
+This implements support for webauthn/FIDO2 Level 2 using the py_webauthn package.
 
-    Check out: https://golb.hplar.ch/2019/08/webauthn.html
-    for some ideas on recovery and adding additional authenticators.
+Check out: https://golb.hplar.ch/2019/08/webauthn.html
+for some ideas on recovery and adding additional authenticators.
 
-    For testing - you can see your YubiKey (or other) resident keys in chrome!
-    chrome://settings/securityKeys
+For testing - you can see your YubiKey (or other) resident keys in chrome!
+chrome://settings/securityKeys
 
-    Observation: if key isn't resident than Chrome for example won't let you use
-    it if it isn't part of allowedCredentials - throw error: referencing:
-    https://www.w3.org/TR/webauthn-2/#sctn-privacy-considerations-client
+Observation: if key isn't resident than Chrome for example won't let you use
+it if it isn't part of allowedCredentials - throw error: referencing:
+https://www.w3.org/TR/webauthn-2/#sctn-privacy-considerations-client
 
-    TODO:
-        - update/add examples to support webauthn
-        - should we universally add endpoint urls to JSON responses?
-        - Add a way to order registered credentials so we can return an ordered list
-          in allowCredentials.
-        - #sctn-usecase-new-device-registration - allow more than one "first" key
-          and have them not necessarily be cross-platform.. add form option?
+TODO:
+    - update/add examples to support webauthn
+    - should we universally add endpoint urls to JSON responses?
+    - Add a way to order registered credentials so we can return an ordered list
+      in allowCredentials.
+    - #sctn-usecase-new-device-registration - allow more than one "first" key
+      and have them not necessarily be cross-platform.. add form option?
 
-    Research:
-        - should we store things like user verified in 'last use'...
-        - By insisting on 2FA if user has registered a webauthn - things
-          get interesting if they try to log in on a different device....
-          How would they register a security key for a new device? They would need
-          some OTHER 2FA? Force them to register a NEW webauthn key?
+Research:
+    - should we store things like user verified in 'last use'...
+    - By insisting on 2FA if user has registered a webauthn - things
+      get interesting if they try to log in on a different device....
+      How would they register a security key for a new device? They would need
+      some OTHER 2FA? Force them to register a NEW webauthn key?
 
 """
 
@@ -77,7 +77,7 @@ except ImportError:  # pragma: no cover
 from .decorators import anonymous_user_required, auth_required, unauth_csrf
 from .forms import (
     Form,
-    Required,
+    RequiredLocalize,
     build_form_from_request,
     build_form,
     get_form_field_label,
@@ -109,8 +109,7 @@ from .utils import (
 if t.TYPE_CHECKING:  # pragma: no cover
     import flask
     from flask.typing import ResponseValue
-    from .core import Security
-    from .datastore import User, WebAuthn
+    from flask_security import Security, UserMixin, WebAuthnMixin
 
 if get_quart_status():  # pragma: no cover
     from quart import redirect
@@ -121,7 +120,7 @@ else:
 class WebAuthnRegisterForm(Form):
     name = StringField(
         get_form_field_xlate(_("Nickname")),
-        validators=[Required(message="WEBAUTHN_NAME_REQUIRED")],
+        validators=[RequiredLocalize(message="WEBAUTHN_NAME_REQUIRED")],
     )
     usage = RadioField(
         get_form_field_xlate(_("Usage")),
@@ -140,6 +139,7 @@ class WebAuthnRegisterForm(Form):
     def validate(self, **kwargs: t.Any) -> bool:
         if not super().validate(**kwargs):
             return False
+        assert isinstance(self.name.errors, list)
         inuse = any([self.name.data == cred.name for cred in current_user.webauthn])
         if inuse:
             msg = get_message("WEBAUTHN_NAME_INUSE", name=self.name.data)[0]
@@ -167,6 +167,10 @@ class WebAuthnRegisterResponseForm(Form):
     def validate(self, **kwargs: t.Any) -> bool:
         if not super().validate(**kwargs):
             return False  # pragma: no cover
+        assert isinstance(self.credential.errors, list)
+        if not self.credential.data:
+            self.credential.errors.append(get_message("API_ERROR")[0])
+            return False
         inuse = any([self.name == cred.name for cred in current_user.webauthn])
         if inuse:
             msg = get_message("WEBAUTHN_NAME_INUSE", name=self.name)[0]
@@ -186,7 +190,7 @@ class WebAuthnRegisterResponseForm(Form):
             self.registration_verification = webauthn.verify_registration_response(
                 credential=reg_cred,
                 expected_challenge=self.challenge.encode(),
-                expected_origin=_security._webauthn_util.origin(),
+                expected_origin=_security.webauthn_util.origin(),
                 expected_rp_id=request.host.split(":")[0],
                 require_user_verification=self.user_verification,
             )
@@ -218,7 +222,7 @@ class WebAuthnSigninForm(Form, NextFormMixin):
     remember = BooleanField(get_form_field_label("remember_me"))
     submit = SubmitField(label=get_form_field_xlate(_("Start")), id="wan_signin")
 
-    user: User | None = None
+    user: UserMixin | None = None
     # set by caller - is this a second factor authentication?
     is_secondary: bool
 
@@ -260,14 +264,18 @@ class WebAuthnSigninResponseForm(Form, NextFormMixin):
     is_verify: bool
     # returned to caller
     authentication_verification: VerifiedAuthentication
-    user: User | None = None
-    cred: WebAuthn | None = None
+    user: UserMixin | None = None
+    cred: WebAuthnMixin | None = None
     # Set to True if this authentication qualifies as 'multi-factor'
     mf_check: bool = False
 
     def validate(self, **kwargs: t.Any) -> bool:
         if not super().validate(**kwargs):
             return False  # pragma: no cover
+        assert isinstance(self.credential.errors, list)
+        if not self.credential.data:
+            self.credential.errors.append(get_message("API_ERROR")[0])
+            return False
         try:
             auth_cred = parse_authentication_credential_json(self.credential.data)
         except (
@@ -327,7 +335,7 @@ class WebAuthnSigninResponseForm(Form, NextFormMixin):
             webauthn.verify_authentication_response,
             credential=auth_cred,
             expected_challenge=self.challenge.encode(),
-            expected_origin=_security._webauthn_util.origin(),
+            expected_origin=_security.webauthn_util.origin(),
             expected_rp_id=request.host.split(":")[0],
             credential_public_key=self.cred.public_key,
             credential_current_sign_count=self.cred.sign_count,
@@ -353,15 +361,19 @@ class WebAuthnSigninResponseForm(Form, NextFormMixin):
 
 
 class WebAuthnDeleteForm(Form):
+    # Change id of name since this shows up on register form that ALSO has a name
+    # element.
     name = StringField(
         get_form_field_xlate(_("Nickname")),
-        validators=[Required(message="WEBAUTHN_NAME_REQUIRED")],
+        validators=[RequiredLocalize(message="WEBAUTHN_NAME_REQUIRED")],
+        id="delete-name",
     )
     submit = SubmitField(label=get_form_field_label("delete"))
 
     def validate(self, **kwargs: t.Any) -> bool:
         if not super().validate(**kwargs):
             return False
+        assert isinstance(self.name.errors, list)
         if not any([self.name.data == cred.name for cred in current_user.webauthn]):
             self.name.errors.append(
                 get_message("WEBAUTHN_NAME_NOT_FOUND", name=self.name.data)[0]
@@ -373,7 +385,7 @@ class WebAuthnDeleteForm(Form):
 class WebAuthnVerifyForm(Form):
     submit = SubmitField(label=get_form_field_label("submit"), id="wan_verify")
 
-    user: User
+    user: UserMixin
 
     def validate(self, **kwargs: t.Any) -> bool:
         if not super().validate(**kwargs):
@@ -404,7 +416,7 @@ def webauthn_register() -> ResponseValue:
     )
 
     if form.validate_on_submit():
-        challenge = _security._webauthn_util.generate_challenge(
+        challenge = _security.webauthn_util.generate_challenge(
             cv("WAN_CHALLENGE_BYTES")
         )
         if not current_user.fs_webauthn_user_handle:
@@ -425,7 +437,7 @@ def webauthn_register() -> ResponseValue:
                 current_user, ["first", "secondary"]
             ),
         )
-        ro = _security._webauthn_util.registration_options(
+        ro = _security.webauthn_util.registration_options(
             current_user, form.usage.data, ro
         )
         credential_options = webauthn.generate_registration_options(**ro)
@@ -464,7 +476,7 @@ def webauthn_register() -> ResponseValue:
         )
 
     current_creds = []
-    cred: WebAuthn
+    cred: WebAuthnMixin
     for cred in current_user.webauthn:
         cl = {
             "name": cred.name,
@@ -549,7 +561,7 @@ def webauthn_register_response(token: str) -> ResponseValue:
         )
         wan_registered.send(
             current_app._get_current_object(),  # type: ignore
-            _async_wrapper=current_app.ensure_sync,
+            _async_wrapper=current_app.ensure_sync,  # type: ignore[arg-type]
             user=current_user,
             name=state["name"],
         )
@@ -568,11 +580,11 @@ def webauthn_register_response(token: str) -> ResponseValue:
     return redirect(url_for_security("wan_register"))
 
 
-def _signin_common(user: User | None, usage: list[str]) -> tuple[t.Any, str]:
+def _signin_common(user: UserMixin | None, usage: list[str]) -> tuple[t.Any, str]:
     """
     Common code between signin and verify - once form has been verified.
     """
-    challenge = _security._webauthn_util.generate_challenge(cv("WAN_CHALLENGE_BYTES"))
+    challenge = _security.webauthn_util.generate_challenge(cv("WAN_CHALLENGE_BYTES"))
 
     # Populate allowedCredentials if identity passed and allowed
     allow_credentials = None
@@ -585,7 +597,7 @@ def _signin_common(user: User | None, usage: list[str]) -> tuple[t.Any, str]:
         timeout=cv("WAN_SIGNIN_TIMEOUT"),
         allow_credentials=allow_credentials,
     )
-    ao = _security._webauthn_util.authentication_options(user, usage, ao)
+    ao = _security.webauthn_util.authentication_options(user, usage, ao)
     options = webauthn.generate_authentication_options(**ao)
 
     # If we ask for UserVerification then we need to check that in the response.
@@ -598,12 +610,14 @@ def _signin_common(user: User | None, usage: list[str]) -> tuple[t.Any, str]:
     }
 
     o_json = json.loads(webauthn.options_to_json(options))
-    state_token = t.cast(str, _security.wan_serializer.dumps(state))
+    state_token = t.cast(  # type: ignore[redundant-cast]
+        str, _security.wan_serializer.dumps(state)
+    )
     return o_json, state_token
 
 
 @anonymous_user_required
-@unauth_csrf(fall_through=True)
+@unauth_csrf()
 def webauthn_signin() -> ResponseValue:
     # This view can be called either as a 'first' authentication or as part of
     # 2FA.
@@ -657,7 +671,7 @@ def webauthn_signin() -> ResponseValue:
     )
 
 
-@unauth_csrf(fall_through=True)
+@unauth_csrf()
 def webauthn_signin_response(token: str) -> ResponseValue:
     is_secondary = all(k in session for k in ["tf_user_id", "tf_state"]) and session[
         "tf_state"
@@ -714,7 +728,7 @@ def webauthn_signin_response(token: str) -> ResponseValue:
             #   - Did this credential provide 2-factor and
             #     is WAN_ALLOW_AS_MULTI_FACTOR set
             #   - Is another 2FA setup?
-            remember_me = form.remember.data if "remember" in form else None
+            remember_me = bool(form.remember.data)
             if form.mf_check and cv("WAN_ALLOW_AS_MULTI_FACTOR"):
                 pass
             else:
@@ -767,7 +781,7 @@ def webauthn_delete() -> ResponseValue:
 
         wan_deleted.send(
             current_app._get_current_object(),  # type: ignore
-            _async_wrapper=current_app.ensure_sync,
+            _async_wrapper=current_app.ensure_sync,  # type: ignore[arg-type]
             user=current_user,
             name=cred.name,
         )
@@ -873,7 +887,7 @@ def webauthn_verify_response(token: str) -> ResponseValue:
     return redirect(url_for_security("wan_verify"))
 
 
-def is_cred_usable(cred: WebAuthn, usage: str | list[str]) -> bool:
+def is_cred_usable(cred: WebAuthnMixin, usage: str | list[str]) -> bool:
     # Return True is cred can be used for the requested usage/verify
     if not isinstance(usage, list):
         usage = [usage]
@@ -881,7 +895,7 @@ def is_cred_usable(cred: WebAuthn, usage: str | list[str]) -> bool:
     return cred.usage in usage
 
 
-def has_webauthn(user: User, usage: str | list[str]) -> bool:
+def has_webauthn(user: UserMixin, usage: str | list[str]) -> bool:
     # Return True if ``user`` has one or more keys with requested usage.
     # Usage: either "first" or "secondary"
     if not isinstance(usage, list):
@@ -894,7 +908,7 @@ def has_webauthn(user: User, usage: str | list[str]) -> bool:
 
 
 def create_credential_list(
-    user: User, usage: list[str]
+    user: UserMixin, usage: list[str]
 ) -> list[PublicKeyCredentialDescriptor]:
     # Return a list of registered credentials - filtered by whether they apply to our
     # authentication state (first or secondary)
@@ -928,13 +942,13 @@ class WebAuthnTfPlugin(TfPluginBase):
         """
         pass
 
-    def get_setup_methods(self, user: User) -> list[str]:
+    def get_setup_methods(self, user: UserMixin) -> list[str]:
         if has_webauthn(user, "secondary"):
             return [_("webauthn")]
         return []
 
     def tf_login(
-        self, user: User, json_payload: dict[str, t.Any], next_loc: str | None
+        self, user: UserMixin, json_payload: dict[str, t.Any], next_loc: str | None
     ) -> ResponseValue:
         session["tf_state"] = "ready"
         if not _security._want_json(request):
