@@ -132,6 +132,9 @@ def app(request):
     # Make this hex_md5 for token tests
     app.config["SECURITY_HASHING_SCHEMES"] = ["hex_md5"]
     app.config["SECURITY_DEPRECATED_HASHING_SCHEMES"] = []
+    app.fs_constructor_args = (
+        dict()
+    )  # allow marks to set items for Security constructor
 
     for opt in [
         "changeable",
@@ -155,6 +158,7 @@ def app(request):
     webauthn_test = marker_getter("webauthn")
     if webauthn_test is not None:
         pytest.importorskip("webauthn")
+        app.fs_constructor_args.update(**webauthn_test.kwargs)
 
     oauthlib_test = marker_getter("oauth")
     if oauthlib_test is not None:
@@ -543,12 +547,46 @@ def fsqlalite_datastore(app, tmpdir, realdburl):
     td()
 
 
-def fsqlalite_setup(app, tmpdir, realdburl):
+@pytest.fixture()
+def fsqlalite_min_datastore(app, tmpdir, realdburl):
+    pytest.importorskip("flask_sqlalchemy_lite")
+    from sqlalchemy.orm import declared_attr, mapped_column, relationship
+    from sqlalchemy import String
+
+    class FsMinUserMixin(UserMixin):
+        # flask_security basic fields
+        id: Mapped[int] = mapped_column(primary_key=True)  # type: ignore
+        email: Mapped[str] = mapped_column(String(255), unique=True)  # type: ignore
+        password: Mapped[str | None] = mapped_column(String(255))  # type: ignore
+        active: Mapped[bool] = mapped_column()  # type: ignore
+        fs_uniquifier: Mapped[str] = mapped_column(  # type: ignore
+            String(64), unique=True
+        )
+
+        @declared_attr
+        def roles(cls):
+            # The first arg is a class name, the backref is a column name
+            return relationship(
+                "Role",
+                secondary="roles_users",
+                back_populates="users",
+            )
+
+    ds, td = fsqlalite_setup(
+        app, tmpdir, realdburl, usermixin=FsMinUserMixin, use_webauthn=False
+    )
+    yield ds
+    td()
+
+
+def fsqlalite_setup(app, tmpdir, realdburl, usermixin=None, use_webauthn=True):
     pytest.importorskip("flask_sqlalchemy_lite")
     from flask_sqlalchemy_lite import SQLAlchemy
     from sqlalchemy.orm import DeclarativeBase, mapped_column
     from flask_security.models import sqla as sqla
 
+    if not usermixin:
+        usermixin = sqla.FsUserMixin
     if realdburl:
         db_url, db_info = _setup_realdb(realdburl)
     else:
@@ -568,10 +606,12 @@ def fsqlalite_setup(app, tmpdir, realdburl):
     class Role(Model, sqla.FsRoleMixin):
         __tablename__ = "role"
 
-    class WebAuthn(Model, sqla.FsWebAuthnMixin):
-        __tablename__ = "webauthn"
+    if use_webauthn:
 
-    class User(Model, sqla.FsUserMixin):
+        class WebAuthn(Model, sqla.FsWebAuthnMixin):
+            __tablename__ = "webauthn"
+
+    class User(Model, usermixin):
         __tablename__ = "user"
         security_number: Mapped[t.Optional[int]] = mapped_column(  # type: ignore
             unique=True
@@ -593,7 +633,10 @@ def fsqlalite_setup(app, tmpdir, realdburl):
             if realdburl:
                 _teardown_realdb(db_info)
 
-    return FSQLALiteUserDatastore(db, User, Role, WebAuthn), tear_down
+    return (
+        FSQLALiteUserDatastore(db, User, Role, WebAuthn if use_webauthn else None),
+        tear_down,
+    )
 
 
 @pytest.fixture()
@@ -900,7 +943,9 @@ def pony_setup(app, tmpdir, realdburl):
 
 @pytest.fixture()
 def client(request, app, sqlalchemy_datastore):
-    app.security = Security(app, datastore=sqlalchemy_datastore)
+    app.security = Security(
+        app, datastore=sqlalchemy_datastore, **app.fs_constructor_args
+    )
     populate_data(app)
     return app.test_client()
 
@@ -940,7 +985,7 @@ def clients(request, app, tmpdir, realdburl, realmongodburl):
     elif request.param == "cl-fsqlalite":
         ds, td = fsqlalite_setup(app, tmpdir, realdburl)
 
-    app.security = Security(app, datastore=ds)
+    app.security = Security(app, datastore=ds, **app.fs_constructor_args)
     populate_data(app)
     if request.param == "cl-peewee":
         # peewee is insistent on a single connection?
@@ -951,7 +996,9 @@ def clients(request, app, tmpdir, realdburl, realmongodburl):
 
 @pytest.fixture()
 def in_app_context(request, app, sqlalchemy_datastore):
-    app.security = Security(app, datastore=sqlalchemy_datastore)
+    app.security = Security(
+        app, datastore=sqlalchemy_datastore, **app.fs_constructor_args
+    )
     with app.app_context():
         yield app
 
@@ -988,7 +1035,7 @@ def datastore(request, app, tmpdir, realdburl, realmongodburl):
         ds, td = sqlalchemy_setup(app, tmpdir, realdburl)
     elif request.param == "sqlalchemy-session":
         if sys.version_info < (3, 10):
-            pytest.skip("requires python3.10 or higher")
+            pytest.skip("sqlalchemy-session requires python3.10 or higher")
         ds, td = sqlalchemy_session_setup(app, tmpdir, realdburl)
     elif request.param == "mongoengine":
         ds, td = mongoengine_setup(app, tmpdir, realmongodburl)
@@ -996,7 +1043,7 @@ def datastore(request, app, tmpdir, realdburl, realmongodburl):
         ds, td = peewee_setup(app, tmpdir, realdburl)
     elif request.param == "pony":
         if sys.version_info > (3, 12):
-            pytest.skip("requires python3.12 or lower")
+            pytest.skip("pony requires python3.12 or lower")
         ds, td = pony_setup(app, tmpdir, realdburl)
     elif request.param == "fsqlalite":
         ds, td = fsqlalite_setup(app, tmpdir, realdburl)
@@ -1022,6 +1069,17 @@ def script_info(app, sqlalchemy_datastore):
     return ScriptInfo(create_app=create_app)
 
 
+@pytest.fixture()
+def script_info_min(app, fsqlalite_min_datastore):
+    from flask.cli import ScriptInfo
+
+    def create_app():
+        app.security = Security(app, datastore=fsqlalite_min_datastore)
+        return app
+
+    return ScriptInfo(create_app=create_app)
+
+
 def pytest_addoption(parser):
     parser.addoption(
         "--realdburl",
